package hideandseek.utils;

import org.bukkit.Material;
import org.bukkit.inventory.InventoryHolder;
import org.bukkit.Bukkit;
import hideandseek.Customization;
import org.bukkit.inventory.Inventory;
import org.bukkit.entity.Player;
import java.util.Collection;
import java.util.List;
import org.bukkit.ChatColor;
import java.util.Iterator;
import org.bukkit.inventory.ItemStack;
import java.util.ArrayList;
import java.util.HashMap;

public class FlexibleSmartInventory
{
    private final String mainName;
    private HashMap<Integer, ArrayList<CustomInventory>> inventories;
    
    public FlexibleSmartInventory(final String var1) {
        this.inventories = new HashMap<Integer, ArrayList<CustomInventory>>();
        this.mainName = var1;
    }
    
    public FlexibleSmartInventory clone() {
        final FlexibleSmartInventory var1 = new FlexibleSmartInventory(this.mainName);
        var1.inventories = (HashMap)this.inventories.clone();
        return var1;
    }
    
    public int addInventory(final String var1, final int var2, final HashMap<Integer, ItemStack> var3) {
        InventorySize var4 = InventorySize.SMALL;
        if (var4.size < var2) {
            var4 = InventorySize.MEDIUM;
        }
        if (var4.size < var2) {
            var4 = InventorySize.LARGE;
        }
        if (var4.size < var2) {
            var4 = InventorySize.MAX;
        }
        return this.addInventory(var1, var4, var3);
    }
    
    public int addInventory(final String var1, final InventorySize var2, final HashMap<Integer, ItemStack> var3) {
        final int var4 = this.inventories.size();
        final CustomInventory var5 = new CustomInventory(this.mainName + (var1.isEmpty() ? "" : (": " + var1)), var2, var3, var4, 0);
        final ArrayList var6 = new ArrayList();
        var6.add(var5);
        if (var4 > 0) {
            var5.addNavigationButtons(false, true);
            this.getLastInventory(var4 - 1).addNavigationButtons(true, false);
        }
        this.inventories.put(var4, var6);
        return var4;
    }
    
    private CustomInventory getLastInventory(final int var1) {
        final ArrayList var2 = this.inventories.get(var1);
        return var2.get(var2.size() - 1);
    }
    
    public ArrayList<PositionData> prepareForItems(final int var1, final int var2) {
        final ArrayList var3 = new ArrayList();
        final ArrayList var4 = this.inventories.get(var1);
        int var5 = var2;
        final HashMap var6 = new HashMap();
        for (final CustomInventory var8 : var4) {
            ArrayList var9;
            for (var9 = var8.getEmptySlots(); var5 > var9.size() && !var8.size.equals(InventorySize.MAX); var9 = var8.getEmptySlots()) {
                var8.expand();
            }
            var6.put(var8.subId, var9);
            var5 -= var9.size();
            if (var5 < 1) {
                break;
            }
        }
        if (var5 > 0) {
            this.getLastInventory(var1).addInventory(InventorySize.SMALL, this.inventories);
            return this.prepareForItems(var1, var2);
        }
        for (final int var10 : var6.keySet()) {
            for (final int var12 : var6.get(var10)) {
                var3.add(new PositionData(var1, var10, var12));
            }
        }
        return var3;
    }
    
    public void removeItem(final int var1, final String var2) {
        for (final CustomInventory var4 : this.inventories.get(var1)) {
            for (final ItemStack var6 : var4.getContents()) {
                if (ChatColor.stripColor(var6.getItemMeta().getDisplayName()).equals(var2)) {
                    var4.removeItem(var6);
                    break;
                }
            }
        }
    }
    
    public List<ItemStack> getContents() {
        final ArrayList var1 = new ArrayList();
        for (int var2 = 0; var2 < this.inventories.size(); ++var2) {
            for (final CustomInventory var4 : this.inventories.get(var2)) {
                var1.addAll(var4.getContents());
            }
        }
        return var1;
    }
    
    public List<ItemStack> getContents(final int var1) {
        final ArrayList var2 = new ArrayList();
        for (final CustomInventory var4 : this.inventories.get(var1)) {
            var2.addAll(var4.getContents());
        }
        return var2;
    }
    
    public void setItem(final int var1, final int var2, final int var3, final ItemStack var4) {
        this.inventories.get(var1).get(var2).setItem(var3, var4);
    }
    
    public PositionData addItem(final ItemStack var1, final int var2) {
        return this.getLastInventory(var2).addItem(var1, var2, this.inventories.get(var2).size() - 1, this.inventories);
    }
    
    public ItemStack getItem(final int var1, final int var2, final int var3) {
        return this.inventories.get(var1).get(var2).inventory.getItem(var3);
    }
    
    public void open(final Player var1) {
        this.open(var1, 0, 0);
    }
    
    public void open(final Player var1, final int var2, final int var3) {
        this.inventories.get(var2).get(var3).open(var1);
    }
    
    public boolean handleScroll(final Player var1, final ItemStack var2, final Inventory var3) {
        if (!Utils.compareItem(var2, Customization.getInstance().items.get("Next")) && !Utils.compareItem(var2, Customization.getInstance().items.get("Previous"))) {
            return false;
        }
        for (int var4 = 0; var4 < this.inventories.size(); ++var4) {
            final List var5 = this.inventories.get(var4);
            for (final CustomInventory var7 : var5) {
                if (var7.inventory.equals(var3)) {
                    int var8;
                    int var9;
                    if (Utils.compareItem(var2, Customization.getInstance().items.get("Next"))) {
                        if (var7.subId + 1 < var5.size()) {
                            var8 = var7.id;
                            var9 = var7.subId + 1;
                        }
                        else {
                            var8 = var7.id + 1;
                            var9 = 0;
                        }
                    }
                    else if (var7.subId == 0) {
                        var8 = var7.id - 1;
                        var9 = this.inventories.get(var4 - 1).size() - 1;
                    }
                    else {
                        var8 = var7.id;
                        var9 = var7.subId - 1;
                    }
                    this.open(var1, var8, var9);
                    break;
                }
            }
        }
        return true;
    }
    
    public String getName() {
        return this.mainName;
    }
    
    public enum InventorySize
    {
        SMALL("SMALL", 0, 27), 
        MEDIUM("MEDIUM", 1, 36), 
        LARGE("LARGE", 2, 45), 
        MAX("MAX", 3, 54);
        
        int size;
        
        private InventorySize(final String name, final int ordinal, final int var3) {
            this.size = var3;
        }
    }
    
    public class CustomInventory
    {
        int id;
        int subId;
        boolean hasNextButton;
        boolean hasBackButton;
        String title;
        Inventory inventory;
        ArrayList<Integer> slots;
        InventorySize size;
        HashMap<Integer, ItemStack> specialItems;
        
        public CustomInventory(final String var2, final InventorySize var3, final HashMap<Integer, ItemStack> var4, final int var5, final int var6) {
            this.setupCustomInventory(var2, var3, var4, var5, var6);
        }
        
        public void setupCustomInventory(final String var1, final InventorySize var2, final HashMap<Integer, ItemStack> var3, final int var4, final int var5) {
            this.title = var1;
            this.size = var2;
            this.specialItems = var3;
            this.id = var4;
            this.subId = var5;
            Utils.cageInventory(this.inventory = Bukkit.createInventory((InventoryHolder)null, var2.size, var1), false);
            if (var3 != null) {
                for (final int var7 : var3.keySet()) {
                    this.inventory.setItem(var7 + this.inventory.getSize() - 9, (ItemStack)var3.get(var7));
                }
            }
            this.slots = new ArrayList<Integer>();
            for (int var8 = 0; var8 < this.inventory.getSize(); ++var8) {
                if (this.inventory.getItem(var8) == null) {
                    this.slots.add(var8);
                }
            }
        }
        
        public void open(final Player var1) {
            var1.openInventory(this.inventory);
        }
        
        public void addNavigationButtons(final boolean var1, final boolean var2) {
            this.hasNextButton = var1;
            this.hasBackButton = var2;
            byte var3 = 0;
            byte var4 = 0;
            if (this.size.equals(InventorySize.SMALL)) {
                var3 = 1;
                var4 = 9;
            }
            else if (this.size.equals(InventorySize.MEDIUM)) {
                var3 = 2;
                var4 = 9;
            }
            else if (this.size.equals(InventorySize.LARGE)) {
                var3 = 3;
                var4 = 9;
            }
            else if (this.size.equals(InventorySize.MAX)) {
                var3 = 2;
                var4 = 18;
            }
            for (int var5 = 0; var5 < var3; ++var5) {
                if (var2) {
                    this.inventory.setItem(var4 + var5 * 9, (ItemStack)Customization.getInstance().items.get("Previous"));
                }
                if (var1) {
                    this.inventory.setItem(var4 + var5 * 9 + 8, (ItemStack)Customization.getInstance().items.get("Next"));
                }
            }
        }
        
        public boolean expand() {
            if (this.size.equals(InventorySize.MAX)) {
                return false;
            }
            InventorySize var1;
            if (this.size.equals(InventorySize.SMALL)) {
                var1 = InventorySize.MEDIUM;
            }
            else if (this.size.equals(InventorySize.MEDIUM)) {
                var1 = InventorySize.LARGE;
            }
            else {
                var1 = InventorySize.MAX;
            }
            final List var2 = this.getContents();
            this.setupCustomInventory(this.title, var1, this.specialItems, this.id, this.subId);
            this.addNavigationButtons(this.hasNextButton, this.hasBackButton);
            int var3 = 0;
            for (final ItemStack var5 : var2) {
                this.inventory.setItem((int)this.slots.get(var3), var5);
                ++var3;
            }
            return true;
        }
        
        public void setItem(final int var1, final ItemStack var2) {
            this.inventory.setItem(var1, var2);
        }
        
        public PositionData addItem(final ItemStack var1, final int var2, final int var3, final HashMap<Integer, ArrayList<CustomInventory>> var4) {
            int var5 = this.getEmptySlot();
            if (var5 != -1) {
                this.inventory.setItem(var5, var1);
                return new PositionData(var2, var3, var5);
            }
            final boolean var6 = this.expand();
            if (var6) {
                var5 = this.getEmptySlot();
                this.inventory.setItem(var5, var1);
                return new PositionData(var2, var3, var5);
            }
            final CustomInventory var7 = this.addInventory(InventorySize.SMALL, var4);
            return var7.addItem(var1, var2, var3 + 1, var4);
        }
        
        public CustomInventory addInventory(final InventorySize var1, final HashMap<Integer, ArrayList<CustomInventory>> var2) {
            final CustomInventory var3 = new CustomInventory(this.title, var1, this.specialItems, this.id, this.subId + 1);
            var3.addNavigationButtons(false, true);
            final ArrayList var4 = var2.get(this.id);
            var4.add(var3);
            var2.put(this.id, var4);
            this.addNavigationButtons(true, false);
            return var3;
        }
        
        public void clear() {
            for (final int var2 : this.slots) {
                this.inventory.setItem(var2, new ItemStack(Material.AIR));
            }
        }
        
        public ArrayList<Integer> getEmptySlots() {
            final ArrayList var1 = new ArrayList();
            for (final int var3 : this.slots) {
                if (this.inventory.getItem(var3) == null) {
                    var1.add(var3);
                }
            }
            return var1;
        }
        
        public int getEmptySlot() {
            for (final int var2 : this.slots) {
                if (this.inventory.getItem(var2) == null) {
                    return var2;
                }
            }
            return -1;
        }
        
        public List<ItemStack> getContents() {
            final ArrayList var1 = new ArrayList();
            for (final int var3 : this.slots) {
                final ItemStack var4 = this.inventory.getItem(var3);
                if (var4 != null) {
                    var1.add(var4);
                }
            }
            return var1;
        }
        
        public void organize() {
            final ArrayList var1 = new ArrayList();
            for (final int var3 : this.slots) {
                final ItemStack var4 = this.inventory.getItem(var3);
                if (var4 != null) {
                    var1.add(var4);
                    this.inventory.setItem(var3, new ItemStack(Material.AIR));
                }
            }
            int var5 = 0;
            final Iterator var6 = var1.iterator();
            while (var6.hasNext()) {
                final ItemStack var4 = var6.next();
                this.inventory.setItem((int)this.slots.get(var5), var4);
                ++var5;
            }
        }
        
        public void removeItem(final ItemStack var1) {
            for (final int var3 : this.slots) {
                final ItemStack var4 = this.inventory.getItem(var3);
                if (var4 != null && var4.equals((Object)var1)) {
                    this.removeItem(var3);
                    break;
                }
            }
        }
        
        public void removeItem(final int var1) {
            this.inventory.setItem(var1, new ItemStack(Material.AIR));
            this.organize();
        }
    }
    
    public class PositionData
    {
        public int id;
        public int subId;
        public int slot;
        
        public PositionData(final int var2, final int var3, final int var4) {
            this.id = var2;
            this.subId = var3;
            this.slot = var4;
        }
    }
}
